import xml
import xml.etree.ElementTree as ET
tree = ET.parse('fun_with_bison/egg.xml')
root = tree.getroot()


#print("LEN", len(child), "TAG", child.tag, "TEXT", child.text, "ATTRIB", child.attrib["name"])


list_of_terminals = []

terminals_lookup = {}
terminals_lookup_rev = {}


for child in root.findall("./grammar/terminals/*"):
    this_terminal = {"token-number":  int(child.attrib["token-number"]),
                     "symbol-number": int(child.attrib["symbol-number"]),
                     "name":              child.attrib["name"]}

    terminals_lookup[child.attrib["name"]] = int(child.attrib["symbol-number"])
    terminals_lookup_rev[int(child.attrib["symbol-number"])] = child.attrib["name"]

    list_of_terminals.append(this_terminal)


print("terminals are", terminals_lookup)

list_of_nonterminals = []
nonterminals_lookup = {}
nonterminals_lookup_rev = {}

for child in root.findall("./grammar/nonterminals/*"):
    this_nonterminal = {"symbol-number": int(child.attrib["symbol-number"]),
                        "name":              child.attrib["name"]}
    list_of_nonterminals.append(this_nonterminal)
    nonterminals_lookup[child.attrib["name"]] = int(child.attrib["symbol-number"])
    nonterminals_lookup_rev[int(child.attrib["symbol-number"])] = child.attrib["name"]


print("nonterminals are", nonterminals_lookup)

def symbol_to_integer(si):
    if si in nonterminals_lookup:
        converted = nonterminals_lookup[si]
    else:
        converted = terminals_lookup[si]

    return converted

list_of_language_rules = {}

renormalized_rule_number = 0
rule_original_to_monotone = {}
rule_monotone_to_original = {}
for child in root.findall("./grammar/rules/*"):
    state_number = int(child.attrib["number"])
    print("SN",renormalized_rule_number, state_number)

    rule_original_to_monotone[state_number] = renormalized_rule_number
    rule_monotone_to_original[renormalized_rule_number] = state_number
    renormalized_rule_number += 1

print(rule_original_to_monotone, "\n", rule_monotone_to_original)







for child in root.findall("./grammar/rules/*"):
    #print("LEN", len(child), "TAG", child.tag, "TEXT", child.text, "ATTRIB", child.attrib)
    rule_number = child.attrib["number"]

    lhs = []

    for x in child.findall("lhs"):
        lhs.append(x.text)

    assert(len(lhs) == 1)
    rule_lhs = nonterminals_lookup[lhs[0]]

    rule_rhs = []
    for x in child.findall("rhs/symbol"):
        rule_rhs.append(symbol_to_integer(x.text))

    the_rule = [rule_lhs, rule_rhs]
    list_of_language_rules[int(rule_number)] = the_rule
    #print("the rule is", the_rule)

print("RULES! ",list_of_language_rules)

the_special_accept_rule = []
for rule_number, rule in list_of_language_rules.items():
    if rule[0] == nonterminals_lookup["$accept"]:
        the_special_accept_rule.append(rule_number)

if(len(the_special_accept_rule) != 1):
    print("assertion fail, unexpected number of special accept rules! Failing!")

acceptrule_number = the_special_accept_rule[0]

#print(acceptrule_number)

list_of_automaton_states = {}

renormalized_state_number = 0
state_original_to_monotone = {}
state_monotone_to_original = {}
for child in root.findall("./automaton/*"):
    state_number = int(child.attrib["number"])
    print("SN",renormalized_state_number, state_number)

    state_original_to_monotone[state_number] = renormalized_state_number
    state_monotone_to_original[renormalized_state_number] = state_number
    renormalized_state_number += 1

print(state_original_to_monotone, "\n", state_monotone_to_original)


for child in root.findall("./automaton/*"):
    state_number = child.attrib["number"]
    #print("STATE NUMBER is", state_number)
    itemset     = []
    transitions = []
    reduces     = []
    #print("len ", len(child), "tag", child.tag, "text", child.text, "attrib", child.attrib)

    for x in child.findall("itemset/"):
        rule_number = rule_original_to_monotone[int(x.attrib["rule-number"])]
        dot_position = int(x.attrib["dot"])
        itemset.append((rule_number,dot_position))

    for x in child.findall("actions/reductions/reduction"):
        symbol = x.attrib["symbol"]
        if (symbol != "$default"):
            symbol = symbol_to_integer(symbol)
        rule   = x.attrib["rule"]
        if (rule == "accept"):
            rule = acceptrule_number
        else:
            rule = rule_original_to_monotone[int(rule)]
        reduces.append((symbol, rule))

    for x in child.findall("actions/transitions/transition"):
        symbol = symbol_to_integer(x.attrib["symbol"])
        type_of_txn = x.attrib["type"]
        state   = state_original_to_monotone[int(x.attrib["state"])]

        transitions.append((type_of_txn,symbol,state))


    list_of_automaton_states[state_original_to_monotone[int(state_number)]] = (itemset, transitions, reduces)





for state_number, x in list_of_automaton_states.items():
    print("state number", state_number, "ITEM SET", x[0],"\n", "TRANSITIONS", x[1],"\n", "REDUCES", x[2],"\n")



# we normalize rule numbers so we can use our convention elsewhere of there not being gaps, starting at 0 etc
# so we can use rule numbers as indices into arrays. Perhaps bison doesn't leave gaps but it's better to
# be sure.


# Now we convert it into a 2-dimensional table:

shift_reduce_table = []


#for rule_number, state in list

